# Docs: https://console.groq.com/docs/quickstart

from groq import Groq
import json
from threading import Lock
from typing import Any
from ..models import Ihunt
from ..stdout import echo
from ..utils import create_prompt, extract_json_from_str, update_data_from_json

API_NAME = "Groq"


# Query: All
def req_groq(ihunt: Ihunt, lock: Lock, data_class: Any) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)
    
    client = Groq(
        api_key=ihunt.apikeys.groq,
    )

    try:
        with lock:
            results = client.chat.completions.create(
                messages=[
                    {
                        "role": "user",
                        "content": create_prompt(ihunt.query.value, data_class),
                    }
                ],
                model="llama3-8b-8192",
                temperature=1,
                top_p=1,
                stream=False,
                # response_format={"type": "json_object"},
                stop=None,
                timeout=ihunt.timeout*2,
            )
            json_str = extract_json_from_str(results.choices[0].message.content)
            if json_str is None:
                raise ValueError("JSON not found in the result.")
            update_data_from_json(ihunt.data, json.loads(json_str))
    except Exception as e:
         echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}...", ihunt.verbose)




# Docs: https://whois.whoisxmlapi.com/documentation/making-requests

import requests
from threading import Lock
from ..models import Ihunt
from ..stdout import echo
from ..utils import is_empty

API_NAME = "WhoisXML"
BASE_URL = "https://www.whoisxmlapi.com/whoisserver/WhoisService"


# Query: Domain
# Return: Info
def req_whoisxml_domain(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL
    headers = {
        "Content-Type": "application/json",
    }
    data = {
        "domainName": ihunt.query.value,
        "outputFormat": "JSON",
        "apiKey": ihunt.apikeys.whoisxml,
    }

    try:
        resp = requests.post(url, headers=headers, json=data, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                d = resp.json()["WhoisRecord"]
                if is_empty(ihunt.data.domain):
                    ihunt.data.domain = d["domainName"]
                if is_empty(ihunt.data.status):
                    ihunt.data.status = [d["status"]]
                # if is_empty(ihunt.data.availability):
                #     ihunt.data.availability = d["domainAvailability"]
                # if is_empty(ihunt.data.ips):
                #     ihunt.data.domain = d["ips"]
                if is_empty(ihunt.data.registrar_name):
                    ihunt.data.registrar_name = d["registryData"]["registrarName"]
                if is_empty(ihunt.data.registrar_iana_id):
                    ihunt.data.registrar_iana_id = d["registryData"]["registrarIANAID"]
                if is_empty(ihunt.data.registrant_organization):
                    ihunt.data.registrant_organization = d["registrant"]["organization"]
                if is_empty(ihunt.data.registrant_country):
                    ihunt.data.registrant_country = d["registrant"]["country"]
                if is_empty(ihunt.data.registrant_country_code):
                    ihunt.data.registrant_country_code = d["registrant"]["countryCode"]
                if is_empty(ihunt.data.registrant_state):
                    ihunt.data.registrant_state = d["registrant"]["state"]
                if is_empty(ihunt.data.registrant_city):
                    ihunt.data.registrant_city = d["registrant"]["city"]
                if is_empty(ihunt.data.admin_organization):
                    ihunt.data.admin_organization = d["administrativeContact"]["organization"]
                if is_empty(ihunt.data.admin_state):
                    ihunt.data.admin_state = d["administrativeContact"]["state"]
                if is_empty(ihunt.data.admin_country):
                    ihunt.data.admin_country = d["administrativeContact"]["country"]
                if is_empty(ihunt.data.admin_country_code):
                    ihunt.data.admin_country_code = d["administrativeContact"]["countryCode"]
                if is_empty(ihunt.data.tech_organization):
                    ihunt.data.tech_organization = d["technicalContact"]["organization"]
                if is_empty(ihunt.data.tech_country):
                    ihunt.data.tech_country = d["technicalContact"]["country"]
                if is_empty(ihunt.data.tech_country_code):
                    ihunt.data.tech_country_code = d["technicalContact"]["countryCode"]
                if is_empty(ihunt.data.tech_state):
                    ihunt.data.tech_state = d["technicalContact"]["state"]
                if is_empty(ihunt.data.name_servers):
                    ihunt.data.name_servers = d["nameServers"]["hostNames"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}...", ihunt.verbose)



# Docs: https://ip-api.com/docs/api:json

import requests
from threading import Lock
from ..models import Ihunt
from ..stdout import echo
from ..utils import is_empty

API_NAME = "ip-api"
BASE_URL = "http://ip-api.com/json"


# Query: IP
# Return: Info
def req_ip_api_ip(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = f"{BASE_URL}/{ihunt.query.value}?fields=status,message,country,countryCode,region,regionName,city,zip,lat,lon,timezone,currency,isp,org,as,reverse,mobile,proxy,hosting,query"

    try:
        resp = requests.get(url, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                d = resp.json()
                if d["status"] != "success":
                    return
                if is_empty(ihunt.data.ip):
                    ihunt.data.ip = d["query"]
                if is_empty(ihunt.data.country_name):
                    ihunt.data.country_name = d["country"]
                if is_empty(ihunt.data.country_code):
                    ihunt.data.country_code = d["countryCode"]
                if is_empty(ihunt.data.region):
                    ihunt.data.region = d["regionName"]
                if is_empty(ihunt.data.city):
                    ihunt.data.city = d["city"]
                if is_empty(ihunt.data.latitude):
                    ihunt.data.latitude = d["latitude"]
                if is_empty(ihunt.data.longitude):
                    ihunt.data.longitude = d["longitude"]
                if is_empty(ihunt.data.timezone):
                    ihunt.data.timezone = d["timezone"]
                if is_empty(ihunt.data.currency):
                    ihunt.data.currency = d["currency"]
                if is_empty(ihunt.data.isp):
                    ihunt.data.isp = d["isp"]
                if is_empty(ihunt.data.domain):
                    ihunt.data.domain = d["reverse"]
                if is_empty(ihunt.data.organization):
                    ihunt.data.organization = d["organization"]
                if is_empty(ihunt.data.is_proxy):
                    ihunt.data.is_proxy = d["proxy"]
                if is_empty(ihunt.data.is_mobile):
                    ihunt.data.is_mobile = d["mobile"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)



import click
from colorama import Fore, Style, init
import itertools
import sys
from threading import Event
import time
from typing import Any


def echo(message: Any, verbose: bool) -> None:
    if verbose:
        click.echo(message)


spinner_chars = itertools.cycle(['|', '/', '-', '\\'])


def spinner(done: Event, text: str) -> None:
    text = f" {text}"

    while not done.is_set():
        sys.stdout.write(Fore.CYAN + next(spinner_chars) + text + Style.RESET_ALL)
        sys.stdout.flush()
        time.sleep(0.1)
        sys.stdout.write('\b' * (len(text) + 1)) # Move the cursor back

    # When finishing, delete the symbol and text.
    sys.stdout.write('\b' * (len(text) + 1))
    sys.stdout.write(' ' * (len(text) + 1))
    sys.stdout.write('\b' * (len(text) + 1))
    sys.stdout.flush()
    


# Docs: https://docs.virustotal.com/reference/domain-info

import requests
from threading import Lock
import base64
import urllib.parse
from ..models import Ihunt
from ..stdout import echo
from ..utils import is_empty

API_NAME = "VirusTotal"
BASE_URL = "https://www.virustotal.com/api/v3"


# Query: Domain
# Return: Info
def req_virustotal_domain(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/domains/{ihunt.query.value}"
    headers = {
        "accept": "application/json",
        "x-apikey": ihunt.apikeys.virustotal,
    }

    try:
        resp = requests.get(url, headers=headers, timeout=ihunt.timeout)
        with lock:
            if resp.status_code == 200:
                d = resp.json()["data"]
                if is_empty(ihunt.data.domain):
                    ihunt.data.domain = d["id"]
                if is_empty(ihunt.data.virustotal_link):
                    ihunt.data.virustotal_link = d["links"]["self"]
                if is_empty(ihunt.data.jarm):
                    ihunt.data.jarm = d["attributes"]["jarm"]
                if is_empty(ihunt.data.virustotal_stats):
                    ihunt.data.virustotal_stats = d["attributes"]["last_analysis_stats"]
                if is_empty(ihunt.data.virustotal_analysis):
                    ihunt.data.virustotal_analysis = d["attributes"]["last_analysis_results"]
                if is_empty(ihunt.data.virustotal_votes):
                    ihunt.data.virustotal_votes = d["attributes"]["total_votes"]
                if is_empty(ihunt.data.https_cert_signature_algorithm):
                    ihunt.data.https_cert_signature_algorithm = d["attributes"]["last_https_certificate"]["cert_signature"]["signature_algorithm"]
                if is_empty(ihunt.data.https_cert_signature):
                    ihunt.data.https_cert_signature = d["attributes"]["last_https_certificate"]["cert_signature"]["signature"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)


# Query: Hash
# Return: Info
def req_virustotal_hash(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/files/{ihunt.query.value}"
    headers = {
        "accept": "application/json",
        "x-apikey": ihunt.apikeys.virustotal,
    }

    try:
        resp = requests.get(url, headers=headers, timeout=ihunt.timeout)
        with lock:
            if resp.status_code == 200:
                d = resp.json()["data"]
                if is_empty(ihunt.data.hash):
                    ihunt.data.hash = d["id"]
                if is_empty(ihunt.data.virustotal_link):
                    ihunt.data.virustotal_link = d["links"]["self"]
                if is_empty(ihunt.data.filetype):
                    ihunt.data.filetype = d["attributes"]["detectiteasy"]["filetype"]
                if is_empty(ihunt.data.filenames):
                    ihunt.data.filenames = d["attributes"]["names"]
                else:
                    # Update filenames array
                    for name in d["attributes"]["names"]:
                        if name not in ihunt.data.filenames:
                            ihunt.data.filenames.append(name)
                if is_empty(ihunt.data.tlsh):
                    ihunt.data.tlsh = d["attributes"]["tlsh"]
                if is_empty(ihunt.data.sha1):
                    ihunt.data.sha1 = d["attributes"]["sha1"]
                if is_empty(ihunt.data.sha256):
                    ihunt.data.sha256 = d["attributes"]["sha256"]
                if is_empty(ihunt.data.md5):
                    ihunt.data.md5 = d["attributes"]["md5"]
                if is_empty(ihunt.data.ssdeep):
                    ihunt.data.ssdeep = d["attributes"]["ssdeep"]
                if is_empty(ihunt.data.vhash):
                    ihunt.data.vhash = d["attributes"]["vhash"]
                if is_empty(ihunt.data.telfhash):
                    ihunt.data.telfhash = d["attributes"]["telfhash"]
                if is_empty(ihunt.data.virustotal_stats):
                    ihunt.data.virustotal_stats = d["attributes"]["last_analysis_stats"]
                if is_empty(ihunt.data.virustotal_analysis):
                    ihunt.data.virustotal_analysis = d["attributes"]["last_analysis_results"]
                if is_empty(ihunt.data.virustotal_votes):
                    ihunt.data.virustotal_votes = d["attributes"]["total_votes"]
                if is_empty(ihunt.data.elf_info):
                    ihunt.data.elf_info = d["attributes"]["elf_info"]
                if is_empty(ihunt.data.pe_info):
                    ihunt.data.pe_info = d["attributes"]["pe_info"]
                if is_empty(ihunt.data.filesize):
                    ihunt.data.filesize = d["attributes"]["size"]
                if is_empty(ihunt.data.magic):
                    ihunt.data.magic = d["attributes"]["magic"]

                if is_empty(ihunt.data.jarm):
                    ihunt.data.jarm = d["attributes"]["jarm"]
                if is_empty(ihunt.data.https_cert_signature_algorithm):
                    ihunt.data.https_cert_signature_algorithm = d["attributes"]["last_https_certificate"]["cert_signature"]["signature_algorithm"]
                if is_empty(ihunt.data.https_cert_signature):
                    ihunt.data.https_cert_signature = d["attributes"]["last_https_certificate"]["cert_signature"]["signature"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)


# Query: IP
# Return: Info
def req_virustotal_ip(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/ip_addresses/{ihunt.query.value}"
    headers = {
        "accept": "application/json",
        "x-apikey": ihunt.apikeys.virustotal,
    }

    try:
        resp = requests.get(url, headers=headers, timeout=ihunt.timeout)
        with lock:
            if resp.status_code == 200:
                d = resp.json()["data"]
                if ihunt.data.ip is None:
                    ihunt.data.ip = d["id"]
                if is_empty(ihunt.data.virustotal_link):
                    ihunt.data.virustotal_link = d["links"]["self"]
                if is_empty(ihunt.data.asn):
                    ihunt.data.asn = d["attributes"]["asn"]
                if is_empty(ihunt.data.virustotal_analysis):
                    ihunt.data.virustotal_analysis = d["attributes"]["last_analysis_results"]
                if is_empty(ihunt.data.jarm):
                    ihunt.data.jarm = d["attributes"]["jarm"]
                if is_empty(ihunt.data.country_code):
                    ihunt.data.country_code = d["attributes"]["country"]
                if is_empty(ihunt.data.net_range):
                    ihunt.data.net_range = d["attributes"]["network"]
                if is_empty(ihunt.data.virustotal_stats):
                    ihunt.data.virustotal_stats = d["attributes"]["last_analysis_stats"]
                if is_empty(ihunt.data.virustotal_votes):
                    ihunt.data.virustotal_votes = d["attributes"]["total_votes"]
                if is_empty(ihunt.data.https_cert_signature_algorithm):
                    ihunt.data.https_cert_signature_algorithm = d["attributes"]["last_https_certificate"]["cert_signature"]["signature_algorithm"]
                if is_empty(ihunt.data.https_cert_signature):
                    ihunt.data.https_cert_signature = d["attributes"]["last_https_certificate"]["cert_signature"]["signature"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)


# Query: URL
# Return: Info
def req_virustotal_url(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    target_url = urllib.parse.quote(base64.b64encode(ihunt.query.value.encode('utf-8')).decode('utf-8'))

    url = BASE_URL + f"/urls/{target_url}"
    headers = {
        "accept": "application/json",
        "x-apikey": ihunt.apikeys.virustotal,
    }

    try:
        resp = requests.get(url, headers=headers, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                d = resp.json()["data"]
                if is_empty(ihunt.data.virustotal_link):
                    ihunt.data.virustotal_link = d["links"]["self"]
                if is_empty(ihunt.data.url):
                    ihunt.data.url = d["attributes"]["url"]
                if is_empty(ihunt.data.virustotal_votes):
                    ihunt.data.virustotal_votes = d["attributes"]["total_votes"]
                if is_empty(ihunt.data.virustotal_stats):
                    ihunt.data.virustotal_stats = d["attributes"]["last_analysis_stats"]
                if is_empty(ihunt.data.virustotal_analysis):
                    ihunt.data.virustotal_analysis = d["attributes"]["last_analysis_results"]
                if is_empty(ihunt.data.virustotal_threat_names):
                    ihunt.data.virustotal_threat_names = d["attributes"]["threat_names"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error {e}", ihunt.verbose)

    echo(f"[*] finished fetching {API_NAME}.", ihunt.verbose)


# Docs: https://ai.google.dev/api?lang=python

import google.generativeai as genai
import json
from threading import Lock
from typing import Any
from ..models import Ihunt
from ..stdout import echo
from ..utils import create_prompt, extract_json_from_str, update_data_from_json

API_NAME = "Gemini"


# Query: All
def req_gemini(ihunt: Ihunt, lock: Lock, data_class: Any) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    genai.configure(api_key=ihunt.apikeys.gemini)
    model = genai.GenerativeModel("gemini-1.5-flash")
   
    try:
        with lock:
            resp = model.generate_content(create_prompt(ihunt.query.value, data_class))
            print(resp.text)
            json_str = extract_json_from_str(resp.text)
            print(json_str)
            if json_str is None:
                raise ValueError("JSON not found in the result.")
            update_data_from_json(ihunt.data, json.loads(json_str))
    except Exception as e:
         echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}...", ihunt.verbose)




from dataclasses import fields
import ipaddress
import json
import re
from typing import Any


def is_empty(data: Any) -> bool:
    if data is None:
        return True
    if isinstance(data, list):
        if len(data) == 0:
            return True
        if all(elem == "" for elem in data):
            return True
    elif isinstance(data, dict):
        if not data:
            return True
    elif isinstance(data, str):
        if data == "":
            return True
    return False


def is_ip_address(s: str) -> bool:
    try:
        ipaddress.ip_address(s)
        return True
    except:
        return False


# Get field names from dataclass.
def get_class_field_names(Any) -> list[str]:
    return [field.name for field in fields(Any)]


# The function is used for creating prompt for AI-related APIs.
def create_prompt(query: str, data_class: Any) -> str:
    return f"""
Give me some information about "{query}". The output rules are below:

- Keep your answers concise.
- The output format is JSON.
- The output is only JSON.
- All values are strings.
- Skip for unknown items, or write empty string "".
- Create list for multiple values.
- Do not use code block.
- The keys of the JSON: {", ".join(get_class_field_names(data_class))}
"""


# The function is used for extracting JSON object from results of LLM APIs.
def extract_json_from_str(text: str) -> str|None:
    json_pattern = re.compile(r'\{.*?\}', re.DOTALL)
    json_strs = json_pattern.findall(text)
    if len(json_strs) == 0:
        return None
    # Find JSON string
    for json_str in json_strs:
        try:
            _ = json.loads(json_str)
            return json_str
        except json.JSONDecodeError:
            continue


# Update Data from given JSON object
def update_data_from_json(data: object, json_obj: dict[str, Any]) -> object:
    for field in fields(data):
        current_value = getattr(data, field.name)
        if field.name in json_obj and is_empty(current_value):
            setattr(data, field.name, json_obj[field.name])


# The function is used when stdout or write to the output file.
def remove_null_values_in_dict(data: dict[Any]) -> dict[Any]:
    new_dict: dict[Any] = {}
    for k, v in data.items():
        if is_empty(v):
            continue
        if isinstance(v, list):
            if len(v) == 0:
                continue
            # Remove empty values
            v = [item for item in v if item != ""]
            # Make unique list (delete duplicates)
            v = list(set(v))
        new_dict[k] = v
    return new_dict



from duckduckgo_search import DDGS
from enum import Enum
import re
from .stdout import echo
from .utils import is_ip_address


class QueryType(Enum):
    DOMAIN  = 1
    EMAIL   = 2
    HASH    = 3
    IP      = 4
    ORG     = 5
    PERSON  = 6
    PHONE   = 7
    URL     = 8
    UNKNOWN = 9


# The function is used for detecting query type.
def identify_querytype_with_duckduckgo(query: str, verbose: bool) -> QueryType:
    echo("[*] Detecting the query type with DuckDuckGo API...", verbose)
    prompt = f"""
Which of the following types can the text "{query}" be categorized into? Please answer just the category type.

Hash
Organization
Person
Phone number
Unknown
"""
    try:
        results = DDGS().chat(f"{prompt}", model='claude-3-haiku')
        if results.lower() == "hash":
            return QueryType.HASH
        elif results.lower() == "organization":
            return QueryType.ORG
        elif results.lower() == "person":
            return QueryType.PERSON
        elif results.lower() == "phone number":
            return QueryType.PHONE
        else:
            return QueryType.UNKNOWN
    except Exception as e:
         echo(f"[x] DuckDuckGo API error: {e}", verbose)
         return QueryType.UNKNOWN


def identify_querytype(query: str, verbose: bool) -> QueryType:
    echo("[*] Identifying the query type...", verbose)

    # If it was not detected by DuckDuckGo, try programatically resolutions.
    domain_pattern = re.compile(r'^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\.)?(?!-)[A-Za-z0-9-]{1,63}(?<!-)\.(?!-)[A-Za-z0-9-]{1,63}(?<!-)$')
    email_pattern = re.compile(r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$')
    url_pattern = re.compile(r'^(https?|ftp|ssh)://[^\s/$.?#].[^\s]*$', re.IGNORECASE)

    if domain_pattern.match(query):
        return QueryType.DOMAIN
    elif email_pattern.match(query):
        return QueryType.EMAIL
    elif is_ip_address(query):
        return QueryType.IP
    elif url_pattern.match(query):
        return QueryType.URL
    
    # Try DuckDuckGo Chat API.
    query_type = identify_querytype_with_duckduckgo(query, verbose)
    return query_type


class Query:
    value: str
    type: QueryType

    def __init__(self, value: str, verbose: bool) -> None:
        self.value = value
        self.type = identify_querytype(value, verbose)

    def type_str(self) -> str:
        if self.type == QueryType.DOMAIN:
            return "Domain"
        elif self.type == QueryType.EMAIL:
            return "Email"
        elif self.type == QueryType.HASH:
            return "Hash"
        elif self.type == QueryType.IP:
            return "IP"
        elif self.type == QueryType.ORG:
            return "Organization"
        elif self.type == QueryType.PERSON:
            return "Person"
        elif self.type == QueryType.PHONE:
            return "Phone"
        elif self.type == QueryType.URL:
            return "URL"
        elif self.type == QueryType.UNKNOWN:
            return "Unknown"





# Docs: https://pypi.org/project/duckduckgo-search/#description

from duckduckgo_search import DDGS
import json
from threading import Lock
from typing import Any
from ..models import Ihunt
from ..stdout import echo
from ..utils import create_prompt, extract_json_from_str, update_data_from_json

API_NAME = "DuckDuckGo"


# Query: All
def req_duckduckgo(ihunt: Ihunt, lock: Lock, data_class: Any) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)
    try:
        with lock:
            results = DDGS().chat(
                create_prompt(ihunt.query.value, data_class),
                model='claude-3-haiku',
                timeout=ihunt.timeout*2,
            )
            json_str = extract_json_from_str(results)
            if json_str is None:
                raise ValueError("JSON not found in the result.")
            update_data_from_json(ihunt.data, json.loads(json_str))
    except Exception as e:
         echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)
    echo(f"[*] Finished fetching {API_NAME}...", ihunt.verbose)




# Docs: https://github.com/ARPSyndicate/docs?tab=readme-ov-file#subdomain-center

import requests
from threading import Lock
from ..models import Ihunt
from ..stdout import echo
from ..utils import is_empty

API_NAME = "SubdomainCenter"
BASE_URL = "https://api.subdomain.center/?domain=google.com"


# Query: Domain
# Return: Subdomains
def req_subdomaincenter_domain(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/?domain={ihunt}"

    try:
        resp = requests.get(url, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                subdomains = resp.json()
                for subdomain in subdomains:
                    if is_empty(ihunt.data.subdomains):
                        ihunt.data.subdomains = [subdomain]
                    else:
                        if subdomain not in ihunt.data.subdomains:
                            ihunt.data.subdomains.append(subdomain)
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}...", ihunt.verbose)



# Docs: https://pulsedive.com/api/

import requests
from threading import Lock
from ..models import Ihunt
from ..stdout import echo
from ..utils import is_empty

API_NAME = "Pulsedive"
BASE_URL = "https://pulsedive.com/api"

# Query: Domain
# Return: Info
def req_pulsedive_domain(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/info.php"
    params = {
        "indicator": "pulsedive.com",
        "pretty": "1",
        "key": ihunt.apikeys.pulsedive,
    }

    try:
        resp = requests.get(url, params=params, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                d = resp.json()
                if is_empty(ihunt.data.ports):
                    ihunt.data.ports = d["attributes"]["port"]
                if is_empty(ihunt.data.protocols):
                    ihunt.data.protocols = d["attributes"]["protocol"]
                if is_empty(ihunt.data.technologies):
                    ihunt.data.technologies = d["attributes"]["technology"]
                if is_empty(ihunt.data.http_headers):
                    ihunt.data.http_headers = d["properties"]["http"]
                if is_empty(ihunt.data.dns):
                    ihunt.data.dns = d["properties"]["dns"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)


# Query: IP
# Return: Info
def req_pulsedive_ip(ihunt: Ihunt, lock: Lock) -> None:
    echo(f"[*] Fetching {API_NAME}...", ihunt.verbose)

    url = BASE_URL + f"/info.php"
    params = {
        "indicator": "pulsedive.com",
        "pretty": "1",
        "key": ihunt.apikeys.pulsedive,
    }

    try:
        resp = requests.get(url, params=params, timeout=ihunt.timeout)
        if resp.status_code == 200:
            with lock:
                d = resp.json()
                if is_empty(ihunt.data.ip):
                    ihunt.data.ip = d["ip"]
                if is_empty(ihunt.data.ports):
                    ihunt.data.ports = d["attributes"]["port"]
                if is_empty(ihunt.data.protocols):
                    ihunt.data.protocols = d["attributes"]["protocol"]
                if is_empty(ihunt.data.technologies):
                    ihunt.data.technologies = d["attributes"]["technology"]
                if is_empty(ihunt.data.http_headers):
                    ihunt.data.http_headers = d["properties"]["http"]
                if is_empty(ihunt.data.dns):
                    ihunt.data.dns = d["properties"]["dns"]
    except Exception as e:
        echo(f"[x] {API_NAME} API error: {e}", ihunt.verbose)

    echo(f"[*] Finished fetching {API_NAME}.", ihunt.verbose)